import { Op } from "sequelize";
import { ERRORENUM } from "../../config/errorEnum";
import { modelMap } from "../../model/modelBind";
import { BizError } from "../../util/bizError";

function analysisParamToWhere(param) {
    let where = {};
    for (let key in param) {// 模糊查询{"字段名":{"%like%":'三'}}
        if (param[key]["%like%"]) {
            where[key] = {[Op.like]:`%${param[key]}%`};
        } else {
            where[key] = param[key];
        }
    }
    return where
}

/**
 * 查询单个数据
 * @param tableModel 表对象
 * @param param 
 * @returns 
 */
export async function selectOneDataByParam(tableModel, column:any, param) {
    let where = analysisParamToWhere(param);
    let data = tableModel.findOne({attributes: column, where});
    return {data};
}


/**
 * 查询多个数据
 * @param tableName 表对象
 * @param param 
 * @returns 
 */
export async function selectDataListByParam(tableModel, column:any, param:any) {
    let where = analysisParamToWhere(param);
    let data = tableModel.findAll({attributes: column, where});
    return {data};
}


/**
 * 分页查询
 * @param tableModel 
 * @param param 
 * @param pageNumber 
 * @param pageSize 
 * @returns 
 */
export async function selectDataListToPageByParam(tableModel, column:any, param:any, pageNumber:number, pageSize:number) {
    pageSize = pageSize || 10;
    let where = analysisParamToWhere(param);
    let data = tableModel.findAll({attributes: column, where, offset:(pageNumber-1)*10, limit:pageSize});
    return {data};
}

export async function selectDataCountByParam(tableModel, param:any) {
    let where = analysisParamToWhere(param);
    let data = tableModel.count({where});
    return {data};
}


export async function associationSelect(tableName:string, param:any) {
    let model = modelMap[tableName];
    if (!model) throw new BizError(ERRORENUM.不存在表);

    let data = await model.aggragateData(param);

    return {data};
}


/**
 * 多表联查
 * @param tableName 
 * @param associatedTable 
 * @param column 
 * @param param 
 * @returns 
 */
export async function selectDataToTableAssociation(tableModel, associatedTable:string, column:any, param:any) {
    let where = analysisParamToWhere(param);
    let data = tableModel.findAll({
        include: [{
            model: associatedTable,
            attributes: column, //指定关联表查询的字段
        }],
        where
    });
    return {data};
}